import io
import os
import sys
import wave
import struct
import threading
from waitress import serve
from flask import Flask, request, stream_with_context

ESPEAK_PATH = os.path.join(os.path.dirname(__file__), 'synthesizer', 'espeak')
if sys.platform.startswith("win"):
    if struct.calcsize("P") * 8 == 32:
        os.environ['PHONEMIZER_ESPEAK_LIBRARY'] = os.path.join(ESPEAK_PATH, 'libespeak-ng-32bit.dll')
    else:
        os.environ['PHONEMIZER_ESPEAK_LIBRARY'] = os.path.join(ESPEAK_PATH, 'libespeak-ng-64bit.dll')

from synthesizer import ChimegeSynthesizer
from synthesizer.tts_normalizer import init_normalizers

SAMPLE_RATE = 22050
VOICES = ['female1', 'female2', 'female3', 'male1', 'male2', 'male3']

TTS_NORMALIZER_CONFIG = {
    'noun_conj': 'noun_conj.json',
    'symbols_file': 'symbols.json',
    'abbreviation_file': 'mongolian_abbreviations.csv',
    'emoji_list': 'emoji_list.json',
    'soft_threshold': 150,
    'hard_threshold': 300
}
TTS_NORMALIZER_OPTIONS = {
    'split_sentences': True,
    'abbreviation_level': 'abbreviation',
    'read_symbols': 'no_temdegt',
    'read_emojis': True,
    'use_phonemizer': True,
    'dont_read_number_n': False,
    'read_legal_number': False,
    'read_roman_number': False,
    'number_chunker': 'default'
}
init_normalizers(TTS_NORMALIZER_CONFIG)
synthesizer = ChimegeSynthesizer(cancel_flag=threading.Event())

app = Flask(__name__)


def create_wav_from_numpy(samples, sample_rate=SAMPLE_RATE):
    mem_out = io.BytesIO()
    with wave.open(mem_out, 'w') as f:
        f.setnchannels(1)
        f.setsampwidth(2)
        f.setframerate(sample_rate)
        f.writeframes(samples.tobytes())
    return mem_out.getvalue()


@app.route("/stream", methods=["POST"])
def nix():
    args = request.get_json()
    voice = args["voice"].strip().lower()
    synthesizer.load_voice_model(voice)
    synthesizer.speed = args["tempo"]
    synthesizer.pitch = args["pitch"]
    synthesizer.volume = args["volume"]

    def ret():
        for audio in synthesizer.synthesize(args["text"], TTS_NORMALIZER_OPTIONS,
                                            chunk_length=args["chunk_length"],
                                            overlap_length=args["overlap_length"]):
            yield audio.tobytes()

    return stream_with_context(ret())


serve(app, host="localhost", port=6969)
