#!/usr/bin/python3
# -*- coding: utf-8 -*-

import os
import sys
import shutil

import argparse
import requests

import pyaudio

import unicodedata as ud

pyaudio_player = pyaudio.PyAudio()
pyaudio_stream = pyaudio_player.open(format=pyaudio.paInt16, channels=1, rate=22050, output=True)


def _get_char_lang(c):
    char_name = ud.name(c)
    if 'LATIN' in char_name:
        return 'en'
    if 'HANGUL' in char_name:
        return 'kr'
    if 'CJK' in char_name:
        return 'jp'
    if 'HIRAGANA' in char_name:
        return 'jp'
    if 'KATAKANA' in char_name:
        return 'jp'
    if 'KATAKANA' in char_name:
        return 'jp'
    if 'CYRILLIC' in char_name:
        return 'mn'
    return 'other'


def segment_text_by_language(text):
    current_lang = 'other'
    current_segment = ''
    segments = []
    for c in text:
        c_lang = _get_char_lang(c)
        if c_lang == 'other':
            current_segment += c
        else:
            if current_lang == 'other':
                current_lang = c_lang
                current_segment += c
            elif current_lang == c_lang:
                current_segment += c
            else:
                segments.append((current_lang, current_segment))
                current_lang = c_lang
                current_segment = '' + c
    if current_lang == 'other':
        current_lang = 'mn'
    segments.append((current_lang, current_segment))
    return segments


if __name__ == '__main__':
    parser = argparse.ArgumentParser(description=__doc__, formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    parser.add_argument('-i', '--voice-id', type=str, default='FEMALE1', help='voice id')
    parser.add_argument('-o', type=str, default='/tmp/mgl.wav', help='dummy option to trick speech dispatcher')
    parser.add_argument('-t', '--text', type=str, required=True, help='text to synthesize')
    parser.add_argument('-s', '--speed', type=float, default=100.0, help='speed')
    parser.add_argument('-p', '--pitch', type=float, default=10, help='pitch')
    parser.add_argument('-v', '--volume', type=float, default=135.0, help='volume')
    args = parser.parse_args()

    # f = open('/tmp/chimege_log.txt', 'at')
    # f.write("%s\n" % " ".join(sys.argv))

    if args.voice_id not in ['FEMALE1', 'FEMALE2', 'FEMALE3', 'MALE1', 'MALE2', 'MALE3']:
        args.voice_id = 'FEMALE2'

    speed = max(-100, min(100, args.speed)) + 100
    chimege_speed = 1.0 + speed * 0.012
    # default is 10 for pitch
    pitch = max(-90, min(90, args.pitch)) - 10
    chimege_pitch = 1.0
    if pitch < 0:
        chimege_pitch = 1.0 + pitch * 0.005
    elif pitch > 0:
        chimege_pitch = 1.0 + pitch * 0.02
    # default 100, min -35!
    volume = max(-35, min(100, args.volume)) + 35
    chimege_volume = 1 / 135 * volume

    pyaudio_player = pyaudio.PyAudio()
    pyaudio_stream = pyaudio_player.open(format=pyaudio.paInt16, channels=1, rate=22050, output=True)

    empty_wav_path = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'empty.wav')
    shutil.copy(empty_wav_path, args.o)

    text = args.text
    text = text.replace(u'\uDCD0', '')
    for segment_lang, segment_text in segment_text_by_language(text):
        if segment_lang == 'mn':
            response = requests.post("http://localhost:6969/stream", json={
                "text": segment_text,
                "voice": args.voice_id,
                "volume": chimege_volume,
                "pitch": chimege_pitch,
                "tempo": chimege_speed,
                "chunk_length": 100,
                "overlap_length": 5
            }, headers={
                "Content-Type": "application/json",
            }, stream=True)

            while True:
                chunk = response.raw.read(1024)
                if len(chunk) == 0:
                    break
                pyaudio_stream.write(chunk)
        else:
            rate = max(-100, min(100, args.speed))
            min_rate = 80
            normal_rate = 170
            max_rate = 449
            if rate < 0:
                speed = normal_rate + (normal_rate - min_rate) * rate / 100
            else:
                speed = normal_rate + (max_rate - normal_rate) * rate / 100
            pitch = max(-100, min(100, args.pitch))
            if pitch < 0:
                pitch = ((pitch + 100.0) * 50) / 100
            else:
                pitch = pitch * (100.0 - 50.0) / 100 + 50
            os.system("espeak-ng -v %s -s %i -p %i \'%s\'" % (segment_lang, int(speed), int(pitch), segment_text))
